# Generated by Django 3.1.3 on 2022-01-14 08:36

import django.core.validators
from django.db import migrations, models
import konova.models.document
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Compensation',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('identifier', models.CharField(blank=True, max_length=1000, null=True)),
                ('title', models.CharField(blank=True, max_length=1000, null=True)),
                ('comment', models.TextField(blank=True, null=True)),
                ('is_cef', models.BooleanField(blank=True, default=False, help_text="Flag if compensation is a 'CEF-Maßnahme'", null=True)),
                ('is_coherence_keeping', models.BooleanField(blank=True, default=False, help_text="Flag if compensation is a 'Kohärenzsicherung'", null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CompensationAction',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('amount', models.FloatField()),
                ('unit', models.CharField(blank=True, choices=[('cm', 'cm'), ('m', 'm'), ('km', 'km'), ('qm', 'm²'), ('ha', 'ha'), ('pcs', 'Pieces')], max_length=100, null=True)),
                ('comment', models.TextField(blank=True, help_text='Additional comment', null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CompensationDocument',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(blank=True, max_length=500, null=True)),
                ('date_of_creation', models.DateField()),
                ('comment', models.TextField()),
                ('file', models.FileField(max_length=1000, upload_to=konova.models.document.generate_document_file_upload_path)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CompensationState',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('surface', models.FloatField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EcoAccount',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('identifier', models.CharField(blank=True, max_length=1000, null=True)),
                ('title', models.CharField(blank=True, max_length=1000, null=True)),
                ('comment', models.TextField(blank=True, null=True)),
                ('access_token', models.CharField(blank=True, help_text='Used for sharing access', max_length=255, null=True)),
                ('deductable_surface', models.FloatField(blank=True, default=0, help_text='Amount of deductable surface - can be lower than the total surface due to deduction limitations', null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EcoAccountDeduction',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('surface', models.FloatField(blank=True, help_text='Amount deducted (m²)', null=True, validators=[django.core.validators.MinValueValidator(limit_value=0.0)])),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EcoAccountDocument',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(blank=True, max_length=500, null=True)),
                ('date_of_creation', models.DateField()),
                ('comment', models.TextField()),
                ('file', models.FileField(max_length=1000, upload_to=konova.models.document.generate_document_file_upload_path)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Payment',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('amount', models.FloatField(validators=[django.core.validators.MinValueValidator(limit_value=0.0)])),
                ('due_on', models.DateField(blank=True, null=True)),
                ('comment', models.TextField(blank=True, help_text="Refers to german money transfer 'Verwendungszweck'", null=True)),
            ],
            options={
                'ordering': ['-amount'],
            },
        ),
    ]
