"""
Author: Michel Peltriaux
Organization: Struktur- und Genehmigungsdirektion Nord, Rhineland-Palatinate, Germany
Contact: ksp-servicestelle@sgdnord.rlp.de
Created on: 19.08.22

"""
from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator

from ema.models import Ema
from konova.decorators import shared_access_required, conservation_office_group_required, login_required_modal
from konova.views.action import AbstractNewCompensationActionView, AbstractEditCompensationActionView, \
    AbstractRemoveCompensationActionView


class NewEmaActionView(AbstractNewCompensationActionView):
    model = Ema
    redirect_url = "ema:detail"

    @method_decorator(login_required_modal)
    @method_decorator(login_required)
    @method_decorator(conservation_office_group_required)
    @method_decorator(shared_access_required(Ema, "id"))
    def dispatch(self, request, *args, **kwargs):
        return super().dispatch(request, *args, **kwargs)


class EditEmaActionView(AbstractEditCompensationActionView):
    model = Ema
    redirect_url = "ema:detail"

    @method_decorator(login_required_modal)
    @method_decorator(login_required)
    @method_decorator(conservation_office_group_required)
    @method_decorator(shared_access_required(Ema, "id"))
    def dispatch(self, request, *args, **kwargs):
        return super().dispatch(request, *args, **kwargs)


class RemoveEmaActionView(AbstractRemoveCompensationActionView):
    model = Ema
    redirect_url = "ema:detail"

    @method_decorator(login_required_modal)
    @method_decorator(login_required)
    @method_decorator(conservation_office_group_required)
    @method_decorator(shared_access_required(Ema, "id"))
    def dispatch(self, request, *args, **kwargs):
        return super().dispatch(request, *args, **kwargs)
